@@ -8,10 +8,9 @@ from lang_trans.arabic import buckwalter
 
 from .entity_mapper import EntityMapper
 from .fact import Fact
-from .parsers import AddressParser, ContactEmailParser, ContactPhoneParserHighPrecision, ContactPhoneParserHighRecall, ContactPhoneParserStrict
+from .parsers import AddressParser, ContactEmailParser, ContactPhoneParserHighPrecision, ContactPhoneParserStrict
 from .russian_transliterator import Transliterate
 
-
 russian_transliterator = Transliterate()
 
 
@@ -30,7 +29,6 @@ class Document:
     FACT_NAME_PHONE_HIGH_PRECISION = "PHONE_high_precision"
     FACT_NAME_PHONE_STRICT = "PHONE_strict"
 
-
     def __init__(
             self,
             original_text: str,
@@ -72,7 +70,6 @@ class Document:
         if self.stanza_document:
             self.words()
 
-
     @property
     def stanza_sentences(self):
         if not self.__stanza_sentences and self.stanza_document:
@@ -80,7 +77,6 @@ class Document:
                 self.__stanza_sentences.append(sentence)
         return self.__stanza_sentences
 
-
     @property
     def stanza_words(self):
         if not self.__stanza_words and self.stanza_document:
@@ -90,7 +86,6 @@ class Document:
 
         return self.__stanza_words
 
-
     @property
     def stanza_entities(self):
         if not self.__stanza_entities:
@@ -98,7 +93,6 @@ class Document:
                 self.__stanza_entities.append(entity)
         return self.__stanza_entities
 
-
     def __get_doc_path(self, field: str) -> str:
         """
         :param field: Whether the doc_path uses the text or lemmas field.
@@ -107,7 +101,6 @@ class Document:
         content = f"{self.doc_path}_mlp.{field}"
         return content
 
-
     def __handle_existing_facts(self):
         """
         Add existing texta_facts inside the document into the private
@@ -119,7 +112,6 @@ class Document:
             for fact in facts:
                 self.add_fact(fact)
 
-
     @staticmethod
     def get_text_from_nested_dict(document: dict, doc_path: str):
         """
@@ -131,9 +123,8 @@ class Document:
         keys = doc_path.split(".")
         return reduce(lambda d, key: d.get(key, None) if isinstance(d, dict) else None, keys, document)
 
-
     @staticmethod
-    def set_text_to_nested_dict(document: dict, doc_path: str, value) -> dict:
+    def edit_doc(document: dict, doc_path: str, value) -> dict:
         """
         :param document: Dictionary object to change.
         :param doc_path: Dot delimited path to the key of a (potentially) nested dictionary,
@@ -141,13 +132,19 @@ class Document:
         :return: Modified dictionary.
         """
         keys = doc_path.split(".")
-        path_to_bottom_layer = keys[:-2]
-        for key in path_to_bottom_layer:
-            document = document.setdefault(key, {})
-        mlp_reserved_key = keys[-1]
-        document[mlp_reserved_key] = value
-        return document
+        throw_away_reference = document
+        for k in keys[:-1]:
+            # This generally only works when using a fully nested dict.
+            second_placeholder = throw_away_reference.setdefault(k, {})
+            # In this scenario, it means we have to shove our new value into the latest layer of the dictionary.
+            if not isinstance(second_placeholder, dict):
+                # Keep the reference to the same layer that we are.
+                throw_away_reference = throw_away_reference
+            else:
+                throw_away_reference = throw_away_reference.setdefault(k, {})
 
+        throw_away_reference[keys[-1]] = value
+        return document
 
     @staticmethod
     def remove_duplicates_with_ignored_keys(duplicates, ignore_keys):
@@ -158,7 +155,6 @@ class Document:
                 yield duplicate
                 seen.add(index)
 
-
     @staticmethod
     def handle_null_values_in_facts(facts: List[dict]):
         container = []
@@ -168,7 +164,6 @@ class Document:
                 container.append(new_fact)
         return container
 
-
     @staticmethod
     def remove_duplicate_facts(facts: List[dict]):
         if facts:
@@ -178,17 +173,14 @@ class Document:
         else:
             return []
 
-
     def facts_to_json(self) -> dict:
         facts = [fact.to_json() for fact in self.__texta_facts]
         unique_facts = Document.remove_duplicate_facts(facts)
         return {"texta_facts": unique_facts}
 
-
     def add_fact(self, fact: Fact):
         self.__texta_facts.append(fact)
 
-
     def fact_spans_to_sent(self):
         """
         Updates fact spans to use sentence-based spans
@@ -217,7 +209,6 @@ class Document:
                 fact.spans = new_spans
                 fact.sent_index = sent_index
 
-
     @staticmethod
     def parse_doc(doc_path: str, document: dict) -> list:
         """
@@ -238,7 +229,6 @@ class Document:
         else:
             return []
 
-
     def document_to_json(self, use_default_doc_path=True) -> dict:
         """
         :param use_default_doc_path: Normal string values will be given the default path for facts but for dictionary input you already have them.
@@ -248,12 +238,11 @@ class Document:
         path_to_mlp = f"{self.doc_path}.{root_key}"
         mlp_result = self.to_json(use_default_doc_path)
 
-        Document.set_text_to_nested_dict(document=self.json_doc, doc_path=path_to_mlp, value=mlp_result["text_mlp"])
-        Document.set_text_to_nested_dict(document=self.json_doc, doc_path="texta_facts", value=mlp_result["texta_facts"])
+        Document.edit_doc(document=self.json_doc, doc_path=path_to_mlp, value=mlp_result["text_mlp"])
+        Document.edit_doc(document=self.json_doc, doc_path="texta_facts", value=mlp_result["texta_facts"])
 
         return self.json_doc
 
-
     def to_json(self, use_default_doc_path=True) -> dict:
         container = dict()
         container["text"] = self.get_words()
@@ -278,19 +267,16 @@ class Document:
                 fact["doc_path"] = "text_mlp.text"
         return {"text_mlp": container, **texta_facts}
 
-
     @staticmethod
     def _clean_lemma(lemma):
         if len(lemma) > 2:
             return lemma.replace("_", "").replace("=", "")
         return lemma
 
-
     def lemmas(self):
         for sent in self.stanza_sentences:
             self.__lemmas.append([self._clean_lemma(word.lemma) if word and word.lemma else "X" for word in sent.words])
 
-
     def get_lemma(self) -> str:
         sentences = []
         if not self.__lemmas:
@@ -302,23 +288,19 @@ class Document:
         else:
             return " ".join(sentences)
 
-
     def sentences(self):
         pass
 
-
     def words(self):
         for sentence in self.stanza_sentences:
             self.__words.append([word.text for word in sentence.words])
 
-
     def get_words(self) -> str:
         if "sentences" in self.analyzers:
             return " \n ".join([" ".join(sent_words) for sent_words in self.__words])
         else:
             return " ".join([" ".join(sent_words) for sent_words in self.__words])
 
-
     def pos_tags(self):
         if "sentences" in self.analyzers:
             for i, sent in enumerate(self.stanza_sentences):
@@ -331,11 +313,9 @@ class Document:
         else:
             self.__pos_tags = [word.upos if word and word.upos and word.upos != "_" else "X" if word.upos == "_" else "X" for word in self.stanza_words]
 
-
     def get_pos_tags(self) -> str:
         return " ".join([a.strip() for a in self.__pos_tags])
 
-
     def word_features(self):
         if "sentences" in self.analyzers:
             for i, sent in enumerate(self.stanza_sentences):
@@ -348,11 +328,9 @@ class Document:
         else:
             self.__word_features = [word.feats if word and word.feats and word.feats != "_" else "X" if word.feats == "_" else "X" for word in self.stanza_words]
 
-
     def get_word_features(self) -> str:
         return " ".join([a.strip() for a in self.__word_features])
 
-
     def entities(self):
         """
         Retrieves list-based entities.
@@ -389,7 +367,6 @@ class Document:
         # declare the entities processed
         self.entities_processed = True
 
-
     def currency_sum(self):
         """
         Extracts currency + sum and sum + currency patterns from text using regexp.
@@ -420,33 +397,28 @@ class Document:
                         )
                         self.__texta_facts.append(new_fact)
 
-
     def emails(self):
         text = self.get_words()
         emails = ContactEmailParser(text).parse()
         self.__texta_facts.extend((email.to_fact(Document.FACT_NAME_EMAIL, self.__get_doc_path("text")) for email in emails))
 
-
     def phone_strict(self):
         text = self.get_words()
         phone_numbers_strict = ContactPhoneParserStrict(text).parse()
         self.__texta_facts.extend(
             (number.to_fact(Document.FACT_NAME_PHONE_STRICT, self.__get_doc_path("text")) for number in phone_numbers_strict))
 
-
     def phone_high_precision(self):
         text = self.get_words()
         phone_numbers_high_precision = ContactPhoneParserHighPrecision(text).parse()
         self.__texta_facts.extend((number.to_fact(Document.FACT_NAME_PHONE_HIGH_PRECISION, self.__get_doc_path("text")) for number in
                                    phone_numbers_high_precision))
 
-
     def addresses(self):
         text = self.get_words()
         addresses = AddressParser(text, self.stanza_entities, self.dominant_language_code).parse()
         self.__texta_facts.extend((addr.to_fact(Document.FACT_NAME_ADDRESS, self.__get_doc_path("text")) for addr in addresses))
 
-
     def transliteration(self):
         if self.dominant_language_code in Document.langs_to_transliterate:
             for word in self.stanza_words:
@@ -456,7 +428,6 @@ class Document:
                     translit_word = self._transliterate_arabic_word(word.text)
                 self.__transliteration.append(translit_word)
 
-
     @staticmethod
     def _transliterate_russian_word(word: str):
         translit_word = russian_transliterator([word.strip()])
@@ -466,7 +437,6 @@ class Document:
             translit_word = word.strip()
         return translit_word
 
-
     @staticmethod
     def _transliterate_arabic_word(word):
         translit_word = buckwalter.transliterate(word.text.strip())
@@ -474,11 +444,9 @@ class Document:
             translit_word = word.text.strip()
         return translit_word
 
-
     def get_transliteration(self) -> str:
         return " ".join(['X' if not a.strip() else a for a in self.__transliteration])
 
-
     def entity_lemmas(self, entity_value):
         lemmas = ""
         splitted = entity_value.split(" ")
@@ -502,7 +470,6 @@ class Document:
                     return word.lemma
         return lemmas
 
-
     def ner(self):
         tokenized_text = self.get_words()
         for entity in self.stanza_entities:
