@@ -15,7 +15,8 @@ from langdetect import detect
 from texta_mlp.document import Document
 from texta_mlp.entity_mapper import EntityMapper
 from texta_mlp.exceptions import CUDAException, LanguageNotSupported
-from texta_mlp.settings import (CUSTOM_NER_MODELS, CUSTOM_NER_MODEL_LANGS, DEFAULT_ANALYZERS, DEFAULT_DOC_PATH_KEY, DEFAULT_LANG_CODES, DEFAULT_OUTPUT_KEY, DEFAULT_RESOURCE_DIR, ENTITY_MAPPER_DATA_URLS, META_KEY, REFRESH_DATA, STANZA_NER_SUPPORT, SUPPORTED_ANALYZERS, USE_GPU)
+from texta_mlp.settings import (CUSTOM_NER_MODELS, CUSTOM_NER_MODEL_LANGS, DEFAULT_ANALYZERS, DEFAULT_DOC_PATH_KEY, DEFAULT_LANG_CODES, DEFAULT_OUTPUT_KEY, DEFAULT_RESOURCE_DIR,
+                                ENTITY_MAPPER_DATA_URLS, META_KEY, REFRESH_DATA, STANZA_NER_SUPPORT, SUPPORTED_ANALYZERS, USE_GPU)
 
 
 class MLP:
@@ -59,7 +60,6 @@ class MLP:
                     raise CUDAException(f"Invalid device id: {gpu_device_id}! Your machine only has {device_count} device(s).")
                 torch.cuda.set_device(gpu_device_id)
 
-
     def prepare_resources(self, refresh_data):
         """
         Prepares all resources for MLP.
@@ -73,7 +73,6 @@ class MLP:
         self.download_stanza_resources(self.resource_dir, self.supported_langs, logger=self.logger)
         self.download_entity_mapper_resources(self.resource_dir, logger=self.logger)
 
-
     @staticmethod
     def download_custom_ner_models(resource_dir: str, logger=None, custom_ner_model_urls: dict = CUSTOM_NER_MODELS, model_langs: list = None):
         """
@@ -92,7 +91,6 @@ class MLP:
                     with open(file_path, "wb") as fh:
                         fh.write(content)
 
-
     @staticmethod
     def download_stanza_resources(resource_dir: str, supported_langs: List[str], logger=None):
         """
@@ -113,7 +111,6 @@ class MLP:
             if not (lang_dir_exists and model_folders_exists):
                 stanza.download(language_code, str(stanza_resource_path))
 
-
     @staticmethod
     def download_entity_mapper_resources(resource_dir: str, logger=None, entity_mapper_urls: tuple = ENTITY_MAPPER_DATA_URLS):
         entity_mapper_resource_path = pathlib.Path(resource_dir) / "entity_mapper"
@@ -128,7 +125,6 @@ class MLP:
                 with open(file_path, "w", encoding="utf8") as fh:
                     fh.write(content)
 
-
     def _load_entity_mapper(self):
         # create Entity Mapper instance
         data_dir = os.path.join(self.resource_dir, "entity_mapper")
@@ -136,7 +132,6 @@ class MLP:
         self.loaded_entity_files = data_files
         return EntityMapper(data_files)
 
-
     @staticmethod
     def normalize_input_text(text: str) -> str:
         """
@@ -152,7 +147,6 @@ class MLP:
         text = text.strip()
         return text
 
-
     def detect_language(self, text: str) -> Optional[str]:
         """
         Detects language of input text.
@@ -167,7 +161,6 @@ class MLP:
             lang = None
         return lang
 
-
     def generate_document(self, raw_text: str, analyzers: List[str], json_object: dict = None, doc_paths=DEFAULT_DOC_PATH_KEY, lang=None, stanza_document=None):
         processed_text = MLP.normalize_input_text(raw_text)
         e = ""
@@ -187,7 +180,7 @@ class MLP:
 
         # Use the pre-given document if it exists, otherwise calculate on own.
         if processed_text and stanza_document is None:
-            document, e = self._get_stanza_document(analysis_lang, processed_text) if processed_text else (None, "")
+            document, e = self._get_stanza_document(analysis_lang, processed_text, analyzers) if processed_text else (None, "", [])
         elif stanza_document and processed_text:
             document = stanza_document
         else:
@@ -207,27 +200,25 @@ class MLP:
         )
         return document
 
-
     @staticmethod
     def _load_analyzers(analyzers, supported_analyzers):
         if analyzers == ["all"]:
             return [analyzer for analyzer in supported_analyzers if analyzer != "all"]
         return [analyzer for analyzer in analyzers if (analyzer in supported_analyzers and analyzer != "all")]
 
-
     def get_entity_mapper(self):
         if self._entity_mapper is None:
             self._entity_mapper = self._load_entity_mapper()
         return self._entity_mapper
 
 
-    def get_stanza_pipeline(self, lang: str):
+    def get_stanza_pipeline(self, lang: str, analyzers: List[str]):
         if lang not in self._stanza_pipelines:
             if lang in self.custom_ner_model_langs:
                 self._stanza_pipelines[lang] = stanza.Pipeline(
                     lang=lang,
                     dir=str(self.stanza_resource_path),
-                    processors=self._get_stanza_processors(lang),
+                    processors=self._get_stanza_processors(lang, analyzers),
                     ner_model_path=f"{self.custom_ner_model_Path}/{lang}",
                     use_gpu=self.use_gpu,
                     logging_level=self.logging_level,
@@ -236,17 +227,17 @@ class MLP:
                 self._stanza_pipelines[lang] = stanza.Pipeline(
                     lang=lang,
                     dir=str(self.stanza_resource_path),
-                    processors=self._get_stanza_processors(lang),
+                    processors=self._get_stanza_processors(lang, analyzers),
                     use_gpu=self.use_gpu,
                     logging_level=self.logging_level,
                 )
         return self._stanza_pipelines[lang]
 
 
-    def _get_stanza_document(self, lang: str, raw_text: str):
+    def _get_stanza_document(self, lang: str, raw_text: str, analyzers: List[str]):
         e = ""
         try:
-            document = self.get_stanza_pipeline(lang)(raw_text)
+            document = self.get_stanza_pipeline(lang, analyzers)(raw_text)
             return document, e
         except KeyError as e:
             raise LanguageNotSupported(f"Language {lang} not supported. Check the list of supported languages.")
@@ -256,22 +247,16 @@ class MLP:
             return None, repr(e)
 
 
-    def _get_stanza_ner(self, lang: str, raw_text: str):
-        pipeline = self.get_stanza_pipeline(lang)(raw_text)
-        return [entity for sentence in pipeline.sentences for entity in sentence.entities]
-
-
     @staticmethod
-    def _get_stanza_processors(lang):
+    def _get_stanza_processors(lang: str, analyzers: List[str]):
         """
         Returns processor options based on language and NER support in Stanza.
         """
-        if lang in STANZA_NER_SUPPORT:
+        if lang in STANZA_NER_SUPPORT and "ner" in analyzers:
             return "tokenize,pos,lemma,ner"
         else:
             return "tokenize,pos,lemma"
 
-
     def process(self, raw_text: str, analyzers: list = DEFAULT_ANALYZERS, lang=None, spans: str = "text"):
         """
         Processes raw text.
@@ -297,7 +282,6 @@ class MLP:
         else:
             return None
 
-
     def lemmatize(self, raw_text: str, lang=None):
         """
         Lemmatizes input text.
@@ -307,14 +291,12 @@ class MLP:
         document = self.process(raw_text, analyzers=["lemmas"], lang=lang)
         return document["text_mlp"]["lemmas"]
 
-
     def __apply_analyzer(self, doc, analyzer):
         try:
             getattr(doc, analyzer)()
         except Exception as e:
             self.logger.exception(e)
 
-
     def __add_meta_to_mlp_output(self, result: dict, field: str, analyzers: List[str], spans: str):
         """Helper function to add meta information to an MLP document that has finished processing."""
         if META_KEY not in result:
@@ -329,7 +311,6 @@ class MLP:
         }
         return result
 
-
     def process_docs(self, docs: List[dict], doc_paths: List[str], analyzers=DEFAULT_ANALYZERS, spans: str = "text"):
         """
         :param docs: Contains tuples with two dicts inside them, the first being the document to be analyzed and the second is the meta information that corresponds to the document for transport purposes later on.
@@ -355,12 +336,15 @@ class MLP:
 
             intermediary = []
             for lang, items in lang_group.items():
-                pipeline = self.get_stanza_pipeline(lang)
+                pipeline = self.get_stanza_pipeline(lang, analyzers)
                 # Create the batch of Stanza Documents to feed into the pipeline.
                 documents = []
                 for item in items:
                     text = item.get("text", "")
                     text = text if text else ""
+                    # Apply preprocessing before starting the analysis as otherwise there will be a mismatch
+                    # because all the tokens will be pulled from the stanza objects.
+                    text = MLP.normalize_input_text(text)
                     documents.append(stanza.Document([], text=text))
                 # Analyze the batch.
                 results = pipeline(documents)
