@@ -1,8 +1,9 @@
 from abc import ABCMeta, abstractmethod
 from typing import Iterator
 
-import regex as re
 import phonenumbers
+import regex as re
+
 from .fact import Fact
 
 
@@ -30,8 +31,8 @@ class AbstractParser(metaclass=ABCMeta):
 
 
 class ContactPhoneParserHighRecall:
-    #High Recall. Gets most of the phone numbers, output has a lot of noisy data.
-    #a bit email-specific
+    # High Recall. Gets most of the phone numbers, output has a lot of noisy data.
+    # a bit email-specific
 
     def __init__(self, input_text, months):
         self.text = input_text
@@ -60,11 +61,13 @@ class ContactPhoneParserHighRecall:
                     replaced = match_string.replace(' ', '').replace('-', '').replace('+', '').replace(')', '').replace('(', '')
                     yield ParseResult(value=replaced, span=numb.span())
 
+
 class ContactPhoneParserHighPrecision:
-    #the output is mainly phone numbers. Excludes more complicated versions.
+    # the output is mainly phone numbers. Excludes more complicated versions.
     def __init__(self, input_text):
         self.text = input_text
 
+
     def parse(self):
         """
         Finds all the phone numbers.
@@ -78,7 +81,8 @@ class ContactPhoneParserHighPrecision:
             match_string = numb.group(0).strip()
             if len(re.sub("[^0-9]", "", match_string)) > 6 and not re.search("\n", match_string):
                 replaced = match_string.replace(' ', '').replace('-', '').replace('+', '')
-                yield ParseResult(value=replaced, span=numb.span()) 
+                yield ParseResult(value=replaced, span=numb.span())
+
 
 class ContactPhoneParserStrict:
     """
@@ -92,18 +96,21 @@ class ContactPhoneParserStrict:
     Also, with sentence "Maksekorraldusele märkida viitenumber 2800049900 ning selgitus .", the "2800049900" is a valid nr in "GB", but not with "EE" or "RU". So be careful when adding country_codes!
 
     """
+
+
     def __init__(self, input_text, country_code=["EE", "RU"]):
         self.text = input_text
         self.country_code = country_code
-        
+
+
     def valid_and_possible(self, number):
 
         try:
             if type(self.country_code) == str:
                 nr = phonenumbers.parse(number, self.country_code)
-                if phonenumbers.is_valid_number(nr): #it’s in an assigned exchange and has the right number of digits
+                if phonenumbers.is_valid_number(nr):  # it’s in an assigned exchange and has the right number of digits
                     return True
-            else: #country_code is a list 
+            else:  # country_code is a list
                 for code in self.country_code:
                     nr = phonenumbers.parse(number, code)
                     if phonenumbers.is_valid_number(nr):
@@ -112,6 +119,7 @@ class ContactPhoneParserStrict:
             return False
         return False
 
+
     def parse(self):
         """
         Finds all the phone numbers.
@@ -125,7 +133,8 @@ class ContactPhoneParserStrict:
             match_string = numb.group(0).strip()
             if len(re.sub("[^0-9]", "", match_string)) > 6 and not re.search("\n", match_string):
                 if self.valid_and_possible(match_string):
-                    yield ParseResult(value=match_string.replace(' ', '').replace('-', '').replace('+', ''), span=numb.span())  
+                    yield ParseResult(value=match_string.replace(' ', '').replace('-', '').replace('+', ''), span=numb.span())
+
 
 class ContactEmailParser(AbstractParser):
     def __init__(self, input_text):
@@ -145,26 +154,6 @@ class ContactEmailParser(AbstractParser):
             yield ParseResult(value=email.group(0).replace('mailto:', '').replace(' ', ''), span=email.span())
 
 
-class ContactEmailNamePairParser(AbstractParser):
-    def __init__(self, input_text):
-        self.text = input_text
-
-
-    def parse(self):
-        """
-        Finds all the emails in the text.
-        Arguments:
-        text -- A string of text containing text to parse emails from.
-        Returns:
-        emails -- a list containing the emails found.
-        """
-        pattern = re.compile(r'(?<=(\s|:|"|^))\p{L}+\s\p{L}+"?\s(< )?([^\s]+\s?\.\s?)?[^\s@]+\s?@\s?[^@.\s]+\s?[.]\s?[^\s]{1,20}')
-        for email in pattern.finditer(self.text):
-            email_splitted = email.group(0).split(" ")
-            if len(email_splitted[1]) > 2 and len(email_splitted[0]) > 2 and email_splitted[0].title() == email_splitted[0] and email_splitted[1].title() == email_splitted[1]:
-                yield ParseResult(value=email.group(0), span=email.span())
-
-
 class AddressParser(AbstractParser):
     def __init__(self, text, stanza_entities, language):
         self.text = text
